Type: Package
Package: arrangements
Title: Fast Generators and Iterators for Permutations, Combinations,
        Integer Partitions and Compositions
Version: 1.1.9
Date: 2020-09-12
Authors@R: 
    person(given = "Randy",
           family = "Lai",
           role = c("aut", "cre"),
           email = "randy.cs.lai@gmail.com")
Description: Fast generators and iterators for permutations, combinations,
    integer partitions and compositions. The arrangements are in
    lexicographical order and generated iteratively in a memory efficient manner. 
    It has been demonstrated that 'arrangements' outperforms most existing
    packages of similar kind. Benchmarks could be found at
    <https://randy3k.github.io/arrangements/articles/benchmark.html>.
License: MIT + file LICENSE
URL: https://github.com/randy3k/arrangements
Depends: R (>= 3.4.0)
Imports: gmp, methods, R6
Suggests: foreach, knitr, rmarkdown, testthat
ByteCompile: yes
Encoding: UTF-8
NeedsCompilation: yes
Roxygen: list(markdown = TRUE)
RoxygenNote: 6.1.1
SystemRequirements: gmp (>= 4.2.3)
Author: Randy Lai [aut, cre]
Maintainer: Randy Lai <randy.cs.lai@gmail.com>
Built: R 4.4.0; aarch64-apple-darwin23.2.0; 2024-04-28 15:10:48 UTC; unix
