# Generated by roxygen2: do not edit by hand

export(Combinations)
export(Compositions)
export(Partitions)
export(Permutations)
export(combinations)
export(compositions)
export(icombinations)
export(icompositions)
export(ipartitions)
export(ipermutations)
export(ncombinations)
export(ncompositions)
export(npartitions)
export(npermutations)
export(partitions)
export(permutations)
import(gmp)
importFrom(R6,R6Class)
importFrom(methods,new)
useDynLib(arrangements, .registration = TRUE, .fixes = "C_")
